
/*	$Id: tixWinInt.h,v 1.1.1.1 2000/05/17 11:08:55 idiscovery Exp $	*/

/*
 * tixWinInt.h
 *
 *	Internal header file for Tix on the Windows platform.
 *
 * Copyright (c) 1996, Expert Interface Technologies
 *
 * See the file "license.terms" for information on usage and redistribution
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 *
 */

#ifndef _TIX_WIN_INT_H_
#define _TIX_WIN_INT_H_

#ifndef _TIX_INT_H_
#include "tixInt.h"
#endif

#endif /* _TIX_WIN_INT_H_ */
